/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.flutter.compdfkit_flutter.common.pdfprovider.provider;

import android.graphics.Bitmap;
import android.graphics.RectF;
import android.graphics.drawable.Drawable;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.bumptech.glide.request.target.CustomTarget;
import com.bumptech.glide.request.transition.Transition;
import com.compdfkit.core.common.CPDFDate;
import com.compdfkit.core.document.CPDFBookmark;
import com.compdfkit.core.document.CPDFDocument;
import com.compdfkit.core.document.CPDFInfo;
import com.compdfkit.core.page.CPDFTextRange;
import com.compdfkit.core.page.CPDFTextSearcher;
import com.compdfkit.core.utils.TTimeUtil;
import com.compdfkit.flutter.compdfkit_flutter.common.pdfprovider.viewer.pdfbookmark.CPDFBookmarkDatas;
import com.compdfkit.flutter.compdfkit_flutter.common.pdfprovider.viewer.pdfinfo.CPDFInfoDatas;
import com.compdfkit.flutter.compdfkit_flutter.common.pdfprovider.viewer.pdfoutline.CPDFOutlineDatas;
import com.compdfkit.flutter.compdfkit_flutter.common.pdfprovider.viewer.pdfsearch.CPDFSearchDatas;
import com.compdfkit.flutter.compdfkit_flutter.common.utils.threadpools.CThreadPoolUtils;
import com.compdfkit.tools.common.utils.glide.CPDFWrapper;
import com.compdfkit.tools.common.utils.glide.GlideApp;
import com.compdfkit.ui.reader.CPDFReaderView;

import org.json.JSONArray;
import org.json.JSONObject;

import java.io.ByteArrayOutputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.flutter.Log;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;


public class CPDFDocumentDataProvider implements CPDFDataProvider {

    CPDFReaderView readerView;

    public CPDFDocumentDataProvider(CPDFReaderView readerView) {
        this.readerView = readerView;
    }

    @Override
    public void updateConfig(String method, Object arguments) {
        switch (method) {
            case Method.SEARCH_BEGIN: {
                Map<String, Object> configs = (Map<String, Object>) arguments;
                int pageIndex = ParamsUtil.getInt(configs, "pageIndex");
                int textRangeIndex = ParamsUtil.getInt(configs, "textRangeIndex");
                Log.e("TextSearcher", "Method.SEARCH_BEGIN -- pageIndex:" + pageIndex + ", textRangeIndex:" + textRangeIndex);
                readerView.getTextSearcher().searchBegin(pageIndex, textRangeIndex);
                break;
            }
            case Method.SEARCH_BACKWARD:
                readerView.getTextSearcher().searchBackward();
                break;
            case Method.SEARCH_FORWARD:
                readerView.getTextSearcher().searchForward();
                break;
            case Method.CANCEL_SEARCH:
                readerView.getTextSearcher().cancelSearch();
                readerView.invalidateAllChildren();
                break;
            default:
                break;
        }
    }

    @Override
    public void resultConfig(MethodCall call, MethodChannel.Result result) {
        CPDFDocument document = readerView.getPDFDocument();
        switch (call.method) {
            case Method.GET_PAGE_COUNT:
                result.success(document.getPageCount());
                break;
            case Method.GET_PAGE_SIZE: {
                RectF size = document.getPageSize((int) call.arguments);
                Map<String, Float> map = new HashMap<>(2);
                map.put("width", size.width());
                map.put("height", size.height());
                result.success(map);
                break;
            }
            case Method.RENDER_PAGE_AT_INDEX: {
                Map<String, Object> arguments = (Map<String, Object>) call.arguments;
                int pageIndex = ParamsUtil.getInt(arguments, "pageIndex");
                int width = ParamsUtil.getInt(arguments, "width");
                int height = ParamsUtil.getInt(arguments, "height");
                CPDFWrapper wrapper = CPDFWrapper.fromDocument(document, pageIndex);
                wrapper.setSize(width, height);
                GlideApp.with(document.getContext()).asBitmap().load(wrapper).diskCacheStrategy(DiskCacheStrategy.NONE).into(new CustomTarget<Bitmap>() {
                    @Override
                    public void onResourceReady(@NonNull Bitmap resource, @Nullable Transition<? super Bitmap> transition) {
                        ByteArrayOutputStream stream = new ByteArrayOutputStream();
                        resource.compress(Bitmap.CompressFormat.JPEG, 100, stream);
                        byte[] byteArray = stream.toByteArray();
                        resource.recycle();
                        ;
                        result.success(byteArray);
                    }

                    @Override
                    public void onLoadCleared(@Nullable Drawable placeholder) {

                    }
                });
                break;
            }
            case Method.GET_OUTLINE:
                result.success(CPDFOutlineDatas.getOutlineList(document, Integer.MAX_VALUE));
                break;
            case Method.GET_INFO:
                result.success(CPDFInfoDatas.getInfo(document));
                break;
            case Method.GET_PERMISSIONS_INFO:
                result.success(CPDFInfoDatas.getPermissionsInfo(document));
                break;
            case Method.SEARCH_KEYWORD: {
                Map<String, Object> configs = (Map<String, Object>) call.arguments;
                String keywords = ParamsUtil.getString(configs, "keywords");
                int searchOptions = ParamsUtil.getInt(configs, "pdfSearchOptions");
                int pageIndex1 = ParamsUtil.getInt(configs, "pageIndex");
                int resultPrefix = ParamsUtil.getInt(configs, "resultPrefix");
                int resultSuffix = ParamsUtil.getInt(configs, "resultSuffix");
                boolean searchAllPage = ParamsUtil.getBoolean(configs, "searchAllPage", false);
                CThreadPoolUtils.getInstance().execute(() -> {
                    String json;
                    if (searchAllPage) {
                        json = CPDFSearchDatas.searchKeywordsWithAllPage(readerView, keywords, searchOptions, resultPrefix, resultSuffix);
                    } else {
                        json = CPDFSearchDatas.searchKeywords(readerView, keywords, searchOptions, pageIndex1, resultPrefix, resultSuffix).toString();
                    }
                    result.success(json);
                });
                break;
            }
            case Method.SEARCH_FIRST: {
                Map<String, Object> configs = (Map<String, Object>) call.arguments;
                String keywords = ParamsUtil.getString(configs, "keywords");
                int searchOptions = ParamsUtil.getInt(configs, "pdfSearchOptions");
                int resultPrefix = ParamsUtil.getInt(configs, "resultPrefix");
                int resultSuffix = ParamsUtil.getInt(configs, "resultSuffix");
                CThreadPoolUtils.getInstance().execute(() -> {
                    String json = CPDFSearchDatas.searchFirst(readerView, keywords, searchOptions, resultPrefix, resultSuffix);
                    result.success(json);
                });
                break;
            }
            case Method.GET_BOOKMARKS: {
                result.success(CPDFBookmarkDatas.getBookmark(document));
                break;
            }
            case Method.ADD_BOOKMARK: {
                Map<String, Object> configs = (Map<String, Object>) call.arguments;
                String title = ParamsUtil.getString(configs, "title");
                int pageIndex = ParamsUtil.getInt(configs, "pageIndex");
                boolean success = document.addBookmark(new CPDFBookmark(
                        pageIndex, title, CPDFDate.toStandardDate(TTimeUtil.getCurrentDate())
                ));
                result.success(success);
                break;
            }
            case Method.REMOVE_BOOKMARK:{
                int pageIndex = ParamsUtil.getValue(call.arguments);
                result.success(document.removeBookmark(pageIndex));
                break;
            }
            case Method.UPDATE_BOOKMARK:{
                Map<String, Object> configs = (Map<String, Object>) call.arguments;
                String title = ParamsUtil.getString(configs, "title");
                int pageIndex = ParamsUtil.getInt(configs, "pageIndex");
                if (document.hasBookmark(pageIndex)) {
                    boolean success = document.getBookmarks().get(pageIndex)
                            .setTitle(title);
                    result.success(success);
                }else {
                    result.success(false);
                }
                break;
            }
            case Method.HAS_BOOKMARK:{
                int pageIndex = ParamsUtil.getValue(call.arguments);
                result.success(document.hasBookmark(pageIndex));
                break;
            }
            default:
                break;
        }
    }

    private static class Method {

        public static final String GET_PAGE_COUNT = "getPageCount";

        public static final String GET_PAGE_SIZE = "getPageSize";

        public static final String RENDER_PAGE_AT_INDEX = "renderPageAtIndex";

        public static final String OPEN_DOCUMENT = "openDocument";

        public static final String GET_OUTLINE = "getOutline";

        public static final String GET_INFO = "getInfo";

        public static final String GET_PERMISSIONS_INFO = "getPermissionsInfo";

        public static final String SEARCH_KEYWORD = "searchKeyword";

        public static final String SEARCH_BEGIN = "searchBegin";

        public static final String CANCEL_SEARCH = "cancelSearch";

        public static final String SEARCH_BACKWARD = "searchBackward";

        public static final String SEARCH_FORWARD = "searchForward";

        public static final String SEARCH_FIRST = "searchFirst";

        public static final String GET_BOOKMARKS = "getBookmarks";

        public static final String ADD_BOOKMARK = "addBookmark";

        public static final String REMOVE_BOOKMARK = "removeBookmark";

        public static final String UPDATE_BOOKMARK = "updateBookmark";
        public static final String HAS_BOOKMARK = "hasBookmark";

    }
}
