/**
 * Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
 * <p>
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 */

package com.compdfkit.flutter.compdfkit_flutter.common.views.cpdfviewctrl;

import android.animation.ObjectAnimator;
import android.content.Context;
import android.net.Uri;
import android.os.Handler;
import android.os.Looper;
import android.util.AttributeSet;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.constraintlayout.widget.ConstraintLayout;

import com.compdfkit.core.annotation.CPDFAnnotation;
import com.compdfkit.core.annotation.form.CPDFWidget;
import com.compdfkit.core.common.CPDFDocumentException;
import com.compdfkit.core.document.CPDFDocument;
import com.compdfkit.core.edit.CPDFEditManager;
import com.compdfkit.flutter.compdfkit_flutter.common.utils.threadpools.CThreadPoolUtils;
import com.compdfkit.ui.reader.CPDFReaderView;
import com.compdfkit.ui.reader.IReaderViewCallback;
import com.compdfkit.ui.reader.OnFocusedTypeChangedListener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.EventChannel;


public class CPDFViewCtrl extends ConstraintLayout implements IReaderViewCallback, OnFocusedTypeChangedListener {

    private CPDFReaderView cPdfReaderView;

    public int currentPageIndex = 0;

    private boolean isScrolling = false;

    private Handler handler = new Handler(Looper.getMainLooper());

    private ObjectAnimator pageIndicatorAnimator = null;

    private List<OnFocusedTypeChangedListener> pdfViewFocusedListenerList = new ArrayList<>();

    private Runnable hideIndicatorRunnable = () -> {
        if (pageIndicatorAnimator != null) {
            isScrolling = false;
            pageIndicatorAnimator.reverse();
        }
    };

    COnEndScrollCallback endScrollCallback = null;

    private EventChannel.EventSink readerViewEventSink = null;

    private BinaryMessenger binaryMessenger;

    private int viewId;

    public CPDFViewCtrl(@NonNull Context context) {
        this(context, null);
    }

    public CPDFViewCtrl(@NonNull Context context, @Nullable AttributeSet attrs) {
        this(context, attrs, 0);
    }

    public CPDFViewCtrl(@NonNull Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        initCPDFReaderView();
    }

    public void registerPlugin(BinaryMessenger messenger, int viewId) {
        this.binaryMessenger = messenger;
        this.viewId = viewId;
        EventChannel readerViewCallbackEventChannel = new EventChannel(binaryMessenger,
                "com.compdfkit.flutter.pdfviewer.readerview.callback." + viewId);
        readerViewCallbackEventChannel.setStreamHandler(new EventChannel.StreamHandler() {
            @Override
            public void onListen(Object arguments, EventChannel.EventSink events) {
                readerViewEventSink = events;
            }

            @Override
            public void onCancel(Object arguments) {

            }
        });
    }


    private void initCPDFReaderView() {
        cPdfReaderView = new CPDFReaderView(getContext());
        cPdfReaderView.setDoublePageMode(false);
        cPdfReaderView.setReaderViewCallback(this);
        cPdfReaderView.setOnFocusedTypeChangedListener(this);
        addView(cPdfReaderView);
    }

    private void setErrorCallback() {
        cPdfReaderView.setPdfErrorMessageCallback(errorId -> {
            switch (errorId) {
                case NO_TEXT_ON_PAGE:
//                    CAlertDialog alertDialog = CAlertDialog.newInstance(
//                            getContext().getString(R.string.tools_warning),
//                            getContext().getString(R.string.tools_scan_pdf_annot_warning)
//                    );
//                    alertDialog.setConfirmClickListener(v -> {
//                        alertDialog.dismiss();
//                    });
//                    if (getContext() instanceof FragmentActivity){
//                        alertDialog.show(((FragmentActivity) getContext()).getSupportFragmentManager(), "alertDialog");
//                    }
                    break;
                case CANNOT_EDIT:
//                    CToastUtil.showToast(getContext(), R.string.tools_can_not_edit);
                    break;
                case NO_EMAIL_APP:
//                    CToastUtil.showToast(getContext(), R.string.tools_reader_view_error_no_email);
                    break;
                case NO_BROWSE_APP:
//                    CToastUtil.showToast(getContext(), R.string.tools_reader_view_error_no_browser);
                    break;
                case INVALID_LINK:
//                    CToastUtil.showToast(getContext(), R.string.tools_reader_view_error_invalid_link);
                    break;
                default:
                    break;
            }
        });
    }

    public void setViewMode(CPDFReaderView.ViewMode viewMode) {
        cPdfReaderView.setViewMode(viewMode);
    }

    /**
     * This method opens a PDF file located at the given file path.
     *
     * @param pdfFilePath The file path of the PDF file to be opened.
     */
    public void openPDF(String pdfFilePath) {
        // Create a new instance of the CPDFDocument class, passing in the current context.
        CPDFDocument cpdfDocument = new CPDFDocument(getContext());
        // Attempt to open the PDF file at the given file path using the open method of the CPDFDocument class.
        CPDFDocument.PDFDocumentError pdfDocumentError = cpdfDocument.open(pdfFilePath);
        setPDFDocument(cpdfDocument, pdfFilePath, pdfDocumentError, null);
    }

    public void openPDF(Uri pdfUri) {
        // Create a new instance of the CPDFDocument class, passing in the current context.
        CPDFDocument cpdfDocument = new CPDFDocument(getContext());
        // Attempt to open the PDF file at the given file path using the open method of the CPDFDocument class.
        CPDFDocument.PDFDocumentError pdfDocumentError = cpdfDocument.open(pdfUri);
        setPDFDocument(cpdfDocument, pdfUri, pdfDocumentError, null);
    }

    public void openPDF(Uri pdfUri, COnOpenPdfFinishCallback openPdfFinishCallback) {
        // Create a new instance of the CPDFDocument class, passing in the current context.
        CPDFDocument cpdfDocument = new CPDFDocument(getContext());
        // Attempt to open the PDF file at the given file path using the open method of the CPDFDocument class.
        CPDFDocument.PDFDocumentError pdfDocumentError = cpdfDocument.open(pdfUri);
        setPDFDocument(cpdfDocument, pdfUri, pdfDocumentError, openPdfFinishCallback);
    }

    private void setPDFDocument(CPDFDocument cpdfDocument, Object pdf, CPDFDocument.PDFDocumentError error, COnOpenPdfFinishCallback openPdfFinishCallback) {
        // Switch on the result of the open method to handle the different possible outcomes.
        switch (error) {
            // If the PDF file was successfully opened, set the PDF document of the CPdfReaderView instance to the opened document.
            case PDFDocumentErrorSuccess:
                cPdfReaderView.setPDFDocument(cpdfDocument);
                cPdfReaderView.setDisplayPageIndex(0);
                if (cPdfReaderView.getEditManager() != null) {
                    cPdfReaderView.getEditManager().disable();
                }
                if (openPdfFinishCallback != null) {
                    openPdfFinishCallback.onOpenPdfFinishCallback();
                }
                setErrorCallback();
                break;
            // If the PDF file requires a password to be opened, do nothing for now.
            case PDFDocumentErrorPassword:
//                CVerifyPasswordDialogFragment verifyPasswordDialogFragment;
//                if (pdf instanceof String) {
//                    verifyPasswordDialogFragment = CVerifyPasswordDialogFragment.newInstance(cpdfDocument, (String) pdf);
//                } else {
//                    verifyPasswordDialogFragment = CVerifyPasswordDialogFragment.newInstance(cpdfDocument, (Uri) pdf);
//                }
//                verifyPasswordDialogFragment.setVerifyCompleteListener(document -> {
//                    cPdfReaderView.setPDFDocument(document);
//                    cPdfReaderView.setDisplayPageIndex(0);
//                    initCPDFSliderBar();
//                    addPageIndicator();
//                    if (cPdfReaderView.getEditManager() != null) {
//                        cPdfReaderView.getEditManager().disable();
//                    }
//                    if (openPdfFinishCallback != null) {
//                        openPdfFinishCallback.onOpenPdfFinishCallback();
//                    }
//                });
//                if (getContext() instanceof FragmentActivity) {
//                    verifyPasswordDialogFragment.show(((FragmentActivity) getContext()).getSupportFragmentManager(), "verifyPwdDialog");
//                }
                break;
            // For all other errors, do nothing for now.
            default:
                break;
        }
    }

    public void savePDF(COnSaveCallback callback) {
        CThreadPoolUtils.getInstance().execute(() -> {
            cPdfReaderView.pauseAllRenderProcess();
            CPDFDocument document = cPdfReaderView.getPDFDocument();
            if (document.hasChanges()) {
                try {
                    boolean success = document.save();
                    if (!success) {
                        document.save(CPDFDocument.PDFDocumentSaveType.PDFDocumentSaveNoIncremental);
                    }
                    if (callback != null) {
                        callback.callback(document.getAbsolutePath(), document.getUri());
                    }
                } catch (CPDFDocumentException e) {
                    e.printStackTrace();
                }
            } else {
                if (callback != null) {
                    callback.callback(document.getAbsolutePath(), document.getUri());
                }
            }
        });
    }

    public void changeFormType(CPDFWidget.WidgetType widgetType) {
        cPdfReaderView.setCurrentFocusedType(CPDFAnnotation.Type.WIDGET);
        cPdfReaderView.setCurrentFocusedFormType(widgetType);
        cPdfReaderView.setTouchMode(CPDFReaderView.TouchMode.ADD_ANNOT);
    }

    public void resetFormType() {
        cPdfReaderView.setTouchMode(CPDFReaderView.TouchMode.BROWSE);
        cPdfReaderView.setCurrentFocusedType(CPDFAnnotation.Type.WIDGET);
        cPdfReaderView.setCurrentFocusedFormType(CPDFWidget.WidgetType.Widget_Unknown);
    }

    public void changeAnnotationType(CPDFAnnotation.Type type) {
        cPdfReaderView.setTouchMode(CPDFReaderView.TouchMode.ADD_ANNOT);
        cPdfReaderView.setCurrentFocusedType(type);
    }

    public void resetAnnotationType() {
        cPdfReaderView.setTouchMode(CPDFReaderView.TouchMode.BROWSE);
        cPdfReaderView.setCurrentFocusedType(CPDFAnnotation.Type.UNKNOWN);
    }

    public void exitEditMode() {
        CPDFEditManager editManager = cPdfReaderView.getEditManager();
        if (editManager != null && editManager.isEditMode()) {
            editManager.endEdit();
        }
    }

    public CPDFReaderView getCPdfReaderView() {
        return cPdfReaderView;
    }

    public void setEndScrollCallback(COnEndScrollCallback callback) {
        this.endScrollCallback = callback;
    }

    @Override
    public void onTypeChanged(CPDFAnnotation.Type type) {
        if (pdfViewFocusedListenerList != null) {
            for (OnFocusedTypeChangedListener onFocusedTypeChangedListener : pdfViewFocusedListenerList) {
                onFocusedTypeChangedListener.onTypeChanged(type);
            }
        }
    }

    @Override
    public void onTapMainDocArea() {
        if (readerViewEventSink != null) {
            Map<String, String> map = new HashMap<>();
            map.put("eventType", "onTapMainDocArea");
            readerViewEventSink.success(map);
        }

    }

    @Override
    public void onMoveToChild(int pageIndex) {
        currentPageIndex = pageIndex;
        if (readerViewEventSink != null) {
            Map<String, Object> map = new HashMap<>();
            map.put("eventType", "onMoveToChild");
            map.put("pageIndex", pageIndex);
            readerViewEventSink.success(map);
        }
    }

    @Override
    public void onEndScroll() {
        if (readerViewEventSink != null) {
            Map<String, String> map = new HashMap<>();
            map.put("eventType", "onScrollEnd");
            readerViewEventSink.success(map);
        }
    }

    @Override
    public void onScrolling() {
        if (readerViewEventSink != null) {
            Map<String, String> map = new HashMap<>();
            map.put("eventType", "onScrolling");
            readerViewEventSink.success(map);
        }
    }

    @Override
    public void onRecordLastJumpPageNum(int pageIndex) {
        if (readerViewEventSink != null) {
            Map<String, Object> map = new HashMap<>();
            map.put("eventType", "onRecordLastJumpPageNum");
            map.put("pageIndex", pageIndex);
            readerViewEventSink.success(map);
        }
    }

    private boolean isHiding = false;

    private void hidePageIndicator() {
        if (!isHiding) {
            isHiding = true;
            handler.postDelayed(hideIndicatorRunnable, 3000);
        }
    }

    public void addOnPDFFocusedTypeChangeListener(OnFocusedTypeChangedListener listener) {
        pdfViewFocusedListenerList.add(listener);
    }

    public interface COnSaveCallback {
        void callback(String filePath, Uri pdfUri);
    }

    public interface COnEndScrollCallback {
        void onEndScrollCallback();
    }

    public interface COnOpenPdfFinishCallback {
        void onOpenPdfFinishCallback();
    }
}
