///
///  Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
///
///  THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
///  AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
///  UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
///  This notice may not be removed from this file.

import 'dart:convert';

import 'package:compdfkit_flutter/core/document/textsearcher/cpdf_search_data.dart';
import 'package:flutter/services.dart';

/// Search for text in documents
class CPDFTextSearcher {
  MethodChannel methodChannel;

  CPDFTextSearcher(this.methodChannel);

  /// Search for keywords in PDF documents.
  /// keywords: The keyword to be searched.
  /// pageIndex: The index of the page to search.
  /// options: Search options.
  /// searchAllPage: Whether to search for keywords on all pages.
  /// resultPrefix: Number of characters to display before the keyword in the returned result text.
  /// resultSuffix: Number of characters to display after the keyword in the returned result text.
  Future<List<List<CPDFSearchData>>> searchKeyword(
      {required String keywords, int pageIndex =0,
      CPDFSearchOptions options = CPDFSearchOptions.PDFSearchCaseInsensitive,
      bool searchAllPage = false,
      int resultPrefix = 20,
      int resultSuffix = 20}) async {
    String resultJson = await methodChannel.invokeMethod('searchKeyword', {
      'keywords': keywords,
      'pdfSearchOptions': options.value,
      'pageIndex': pageIndex,
      'resultPrefix': resultPrefix,
      'resultSuffix': resultSuffix,
      'searchAllPage': searchAllPage
    });
    List<dynamic> list = json.decode(resultJson);
    return list.map((e) {
      List<dynamic> item = e;
      return item.map((map) {
        var a = map;
        return CPDFSearchData.formJson(a);
      }).toList();
    }).toList();
  }

  /// Search from the first page of the document, looking for the first matching keyword
  Future<CPDFSearchData?> searchFirst(
      {required String keywords,
      CPDFSearchOptions options = CPDFSearchOptions.PDFSearchCaseInsensitive,
      int resultPrefix = 20,
      int resultSuffix = 20}) async {
    String? resultJson = await methodChannel.invokeMethod('searchFirst', {
      'keywords': keywords,
      'pdfSearchOptions': options.value,
      'resultPrefix': resultPrefix,
      'resultSuffix': resultSuffix,
    });
    if (resultJson != null) {
      var da = json.decode(resultJson);
      CPDFSearchData data = CPDFSearchData.formJson(da);
      return data;
    } else {
      return null;
    }
  }

  /// Highlight the search keywords of the specified page in the interface
  /// 'pageIndex', 'textRangeIndex' parameters You can query keywords through the
  /// 'searchKeyword' and 'searchFirst' methods, and get the parameters in 'CPDFSearchData'
  void searchBegin({required int pageIndex, required int textRangeIndex}) {
    methodChannel.invokeMethod('searchBegin',
        {'pageIndex': pageIndex, 'textRangeIndex': textRangeIndex});
  }

  /// Cancel the search and cancel the highlighted keywords in the view
  void cancelSearch() {
    methodChannel.invokeMethod('cancelSearch');
  }

  /// Highlight previous searched keywords
  void searchBackward() {
    methodChannel.invokeMethod('searchBackward');
  }

  /// Highlight the next searched keyword
  void searchForward() {
    methodChannel.invokeMethod('searchForward');
  }
}

enum CPDFSearchOptions {
  PDFSearchCaseInsensitive(value: 0),
  PDFSearchCaseSensitive(value: 1),
  PDFSearchMatchWholeWord(value: 2),
  PDFSearchConsecutive(value: 4);

  final int value;

  const CPDFSearchOptions({required this.value});
}
