///
///  Copyright © 2014-2023 PDF Technologies, Inc. All Rights Reserved.
///
///  THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
///  AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
///  UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
///  This notice may not be removed from this file.

import 'package:compdfkit_flutter/core/document/cpdf_document.dart';
import 'package:compdfkit_flutter/core/document/textsearcher/cpdf_search_data.dart';
import 'package:compdfkit_flutter/theme/colors.dart';
import 'package:flutter/material.dart';

import '../../../common/util/Strings.dart';
import '../../common/views/cpdf_scaffold.dart';
import '../../common/views/cpdf_tool_bar.dart';

class CPDFSearchResultListPage extends StatefulWidget {
  final bool isDark;

  final CPDFDocument document;

  final String keywords;

  const CPDFSearchResultListPage(
      {Key? key,
      this.isDark = false,
      required this.document,
      required this.keywords})
      : super(key: key);

  @override
  State<CPDFSearchResultListPage> createState() =>
      _CPDFSearchResultListPageState();
}

class _CPDFSearchResultListPageState extends State<CPDFSearchResultListPage> {
  @override
  Widget build(BuildContext context) {
    return CPDFScaffold(
        isDark: widget.isDark,
        appBar: CPDFToolbar.normal(
          context: context,
          titleText: Strings.Results,
          leadingOnPressed: () {
            Navigator.pop(context);
          },
        ),
        body: _searchResultWidget());
  }

  Widget _searchResultWidget() {
    return FutureBuilder(
        future: _requestSearchData(),
        builder: (context, snapshot) {
          if (snapshot.connectionState == ConnectionState.done &&
              snapshot.hasData) {
            List<CPDFSearchData> list = snapshot.data!;
            return Column(
              children: [
                Container(
                  alignment: Alignment.centerLeft,
                  height: 28,
                  padding: const EdgeInsets.symmetric(horizontal: 16),
                  color: Theme.of(context).colorScheme.secondaryContainer,
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      Text(
                        '${list.length} ${Strings.Results}',
                        style: Theme.of(context).textTheme.labelMedium,
                      ),
                      Text(Strings.Page,
                          style: Theme.of(context).textTheme.bodySmall)
                    ],
                  ),
                ),
                Expanded(child: ListView.builder(
                    itemCount: list.length,
                    itemBuilder: (context, index) {
                  CPDFSearchData data = list[index];
                  if(data.isHead){
                    return _head(data);
                  }else{
                    return _searchItem(data);
                  }
                }))
              ],
            );
          } else {
            return _loadingWidget();
          }
        });
  }

  Future<List<CPDFSearchData>> _requestSearchData() async {
    List<List<CPDFSearchData>> data = await widget.document.textSearcher
        .searchKeyword(keywords: widget.keywords, searchAllPage: true,pageIndex: 1, resultPrefix: 20, resultSuffix: 20);
    List<CPDFSearchData> list = List.empty(growable: true);
    for (var value in data) {
      list.add(CPDFSearchData.head(true, value.first.pageIndex));
      list.addAll(value);
    }
    return list;
  }

  Widget _head(CPDFSearchData data) {
    return Container(
      alignment: Alignment.centerRight,
      height: 28,
      padding: const EdgeInsets.symmetric(horizontal: 16),
      color: Theme.of(context).colorScheme.inversePrimary,
      child: Text((data.pageIndex + 1).toString(),
          style: Theme.of(context).textTheme.bodySmall),
    );
  }

  Widget _searchItem(CPDFSearchData data) {
    return Ink(
      color: widget.isDark ? CPDFColors.backgroundDark : CPDFColors.backgroundLight,
      child: InkWell(
          onTap: () {
            Navigator.pop(context, data);
          },
          child: Padding(
              padding: const EdgeInsets.all(16),
              child: Text.rich(
                TextSpan(children: [
                  TextSpan(text: data.text.substring(0, data.textStartIndex)),
                  TextSpan(
                      text: data.text
                          .substring(data.textStartIndex, data.textEndIndex),
                      style: const TextStyle(backgroundColor: Color(0xFFFFEC66))),
                  TextSpan(
                      text: data.text
                          .substring(data.textEndIndex, data.text.length)),
                ]),
                maxLines: 3,
                style: Theme.of(context).textTheme.bodyMedium?.copyWith(color: widget.isDark ? Colors.white : const Color(0xFF666666)),
              ))),
    );
  }

  Widget _loadingWidget() {
    return Center(
      child: CircularProgressIndicator(
          color: Theme.of(context).colorScheme.primaryContainer),
    );
  }
}
